function F = bilinear(arg1,arg2,arg3, arg4, arg5, ExtrapVal)

//LINEAR 2-D bilinear data interpolation.
//   ZI = LINEAR(X,Y,Z,XI,YI) uses bilinear interpolation to
//   find ZI, the values of the underlying 2-D function in Z at the points
//   in matrices XI and YI.  Matrices X and Y specify the points at which 
//   the data Z is given.  X and Y can also be vectors specifying the 
//   abscissae for the matrix Z as for MESHGRID. In both cases, X
//   and Y must be equally spaced and monotonic.
//
//   Values of NaN are returned in ZI for values of XI and YI that are 
//   outside of the range of X and Y.
//
//   If XI and YI are vectors, LINEAR returns vector ZI containing
//   the interpolated values at the corresponding points (XI,YI).
//   ZI = LINEAR(Z,XI,YI) assumes X = 1:N and Y = 1:M, where
//   [M,N] = SIZE(Z).
//
//   ZI = LINEAR(Z,NTIMES) returns the matrix Z expanded by interleaving
//   bilinear interpolates between every element, working recursively
//   for NTIMES.  LINEAR(Z) is the same as LINEAR(Z,1).
//
//   This function needs about 4 times SIZE(XI) memory to be available.
//
[nargout,nargin]=argn(0);
if nargin==2 then // linear(z), Expand Z
  [nrows,ncols] = size(arg1);
  s = 1:.5:ncols;
  t = (1:.5:nrows)';
  [sizs1,sizs2] = size(s);
  [sizt1,sizt2] = size(t);
  s = s(ones(sizt1,sizt2),:);
  t = t(:,ones(sizs1,sizs2));
elseif nargin==3 then // linear(z,n), Expand Z n times
  [nrows,ncols] = size(arg1);
  ntimes = floor(arg2);
  s = 1:1/(2^ntimes):ncols;
  t = (1:1/(2^ntimes):nrows)'; 
  [sizs1,sizs2] = size(s);
  [sizt1,sizt2] = size(t);
  s = s(ones(sizt1,sizt2),:);
  t = t(:,ones(sizs1,sizs2));
elseif nargin==4 then
    [nrows,ncols]=size(arg1);
    s=arg2; 
    t=arg3;
    ExtrapVal=arg4;
elseif nargin==5 then
    error('Wrong number of argumnets!');
elseif nargin==6 then
    [nrows,ncols] = size(arg3);
  mx = prod(size(arg1)); my = prod(size(arg2));
  if or([mx my] ~= [ncols nrows]) & ~isequal(size(arg1),size(arg2),size(arg3))
    error('The lengths of the X and Y vectors must match Z.');
  end
  if or([nrows ncols]<[2 2])
    error('Z must be at least 2-by-2.'); 
  end
  s = 1 + (arg4-arg1(1))/(arg1(mx)-arg1(1))*(ncols-1);
  t = 1 + (arg5-arg2(1))/(arg2(my)-arg2(1))*(nrows-1);
  
end
    

if or([nrows ncols]<[2 2])
  error('Z must be at least 2-by-2.'); 
end

if ~isequal(size(s),size(t)),
  error('XI and YI must be the same size.');
end

// Check for out of range values of s and set to 1
sout = find((s<1)|(s>ncols));
if length(sout)>0 then
     s(sout) = ones(size(sout,1),size(sout,2));
end

// Check for out of range values of t and set to 1
tout = find((t<1)|(t>nrows));
if length(tout)>0 then
    t(tout) = ones(size(tout,1),size(tout,2));
end

// Matrix element indexing
ndx = floor(t)+floor(s-1)*nrows;

// Compute intepolation parameters, check for boundary value.
if isempty(s) then
     d = s;
  else 
      d = find(s==ncols);
end
s(:) = (s - floor(s));
if length(d)>0 then
     s(d) = s(d)+1; 
     ndx(d) = ndx(d)-nrows; 
 end

// Compute intepolation parameters to check boundary value.
if isempty(t) then
     d = t;
  else 
      d = find(t==nrows);
 end
t(:) = (t - floor(t));
if length(d)>0 then
     t(d) = t(d)+1; 
     ndx(d) = ndx(d)-1; 
 end
d = [];
[sx,sy]=size(s);
if nargin==6 then
    F1=( matrix(arg3(ndx),sx,sy).*(1-t) + matrix(arg3(ndx+1),sx,sy).*t ).*(1-s);
    F2=( matrix(arg3(ndx+nrows),sx,sy).*(1-t) + matrix(arg3(ndx+(nrows+1)),sx,sy).*t ).*s;
else
    F1=( matrix(arg1(ndx),sx,sy).*(1-t) + matrix(arg1(ndx+1),sx,sy).*t ).*(1-s);
    F2=( matrix(arg1(ndx+nrows),sx,sy).*(1-t) + matrix(arg1(ndx+(nrows+1)),sx,sy).*t ).*s; 
end
F=F1+F2;
if length(sout)>0, F(sout) = ExtrapVal; end
if length(tout)>0, F(tout) = ExtrapVal; end
endfunction

